'use strict';

const dotEnv = require('dotenv');
const {safeLoad} = require('js-yaml');
const {join} = require('path');
const {lowerCase, clone, merge} = require('lodash');
const {existsSync, readFileSync} = require('fs');

// Load .env file in projects root directory
dotEnv.config();

// Get NODE_ENV to determine which configuration file to load
const NODE_ENV = process.env.NODE_ENV || 'development';

/**
 * Sets configuration path
 * Arguments to function are passed to path.join to build out config path
 * @example
 * setConfigPath(__dirname, 'config');
 */
function setConfigPath () {
  const configPath = join.apply(join, arguments);
  if (existsSync(configPath)) {
    const envCopy = clone(process.env);
    mergeFileWithEnv(configPath, 'default');
    mergeFileWithEnv(configPath, lowerCase(NODE_ENV));
    mergeWithEnv(envCopy);
  } else {
    throw new Error('Config path to load does not exist.');
  }
}

/**
 * Get configuration value
 * @param {String} path - configuration variable name
 * @param {Function} [transform] - optional transformation function, if not passed in string will be returned
 * @returns {String|*}
 */
function getConfig (path, transform) {
  if (transform) return transform(process.env[path]);
  return process.env[path];
}

/**
 * Helper function to parse coma delimited strings
 * @param {String} string
 */
function arrayConfig (string) {
  return string.split(',');
}

function booleanConfig (string) {
  return string === 'true';
}

function mergeFileWithEnv (configPath, fileName) {
  const yml = join(configPath, `${fileName}.yml`);
  if (existsSync(yml)) {
    return mergeWithEnv(safeLoad(readFileSync(yml, 'utf8')));
  }
}

function mergeWithEnv (config) {
  return merge(process.env, config);
}

// Exports
exports.setConfigPath = setConfigPath;
exports.getConfig = getConfig;
exports.arrayConfig = arrayConfig;
exports.booleanConfig = booleanConfig;
